@@ -1,10 +1,13 @@
 import socket
+import sys
 import threading
 from enum import Enum
 from .hpacket import HPacket
 from .hmessage import HMessage, Direction
 import json
 
+MINIMUM_GEARTH_VERSION = "1.4.1"
+
 
 class INCOMING_MESSAGES(Enum):
     ON_DOUBLE_CLICK = 1
@@ -62,7 +65,10 @@ def get_argument(args, flags):
 
 
 class Extension:
-    def __init__(self, extension_info, args, extension_settings=None):
+    def __init__(self, extension_info, args, extension_settings=None, silent=False):
+        if not silent:
+            print("WARNING: This version of G-Python requires G-Earth >= {}".format(MINIMUM_GEARTH_VERSION), file=sys.stderr)
+
         extension_settings = fill_settings(extension_settings, EXTENSION_SETTINGS_DEFAULT)
 
         if get_argument(args, PORT_FLAG) is None:
@@ -86,7 +92,7 @@ class Extension:
         self._extension_settings = extension_settings
 
         self.connection_info = None
-        self.harble_api = None
+        self.packet_infos = None
 
         self.__start_barrier = threading.Barrier(2)
         self.__start_lock = threading.Lock()
@@ -148,15 +154,16 @@ class Extension:
                 habbo_packet.reset()
 
             header_id = habbo_packet.header_id()
-            hash, name = None, None
-            if self.harble_api is not None and header_id in self.harble_api[habbo_message.direction]:
-                hash = self.harble_api[habbo_message.direction][header_id]['Hash']
-                name = self.harble_api[habbo_message.direction][header_id]['Name']
-                if name == '' or name == 'null':
-                    name = None
+            potential_intercept_ids = {header_id}
+            if self.packet_infos is not None and header_id in self.packet_infos[habbo_message.direction]:
+                for elem in self.packet_infos[habbo_message.direction][header_id]:
+                    if elem['Name'] is not None:
+                        potential_intercept_ids.add(elem['Name'])
+                    if elem['Hash'] is not None:
+                        potential_intercept_ids.add(elem['Hash'])
 
-            for id in [header_id, hash, name]:
-                if id is not None and id in self.__intercept_listeners[habbo_message.direction]:
+            for id in potential_intercept_ids:
+                if id in self.__intercept_listeners[habbo_message.direction]:
                     for func in self.__intercept_listeners[habbo_message.direction][id]:
                         func(habbo_message)
                         habbo_packet.reset()
@@ -195,22 +202,22 @@ class Extension:
                 self.__send_to_stream(response)
 
             elif message_type == INCOMING_MESSAGES.CONNECTION_START:
-                host = packet.read_string()
-                port = packet.read_int()
-                hotel_version = packet.read_string()
-                harble_messages_path = packet.read_string()
-                self.connection_info = {'host': host, 'port': port, 'hotel_version': hotel_version,
-                                        'harble_messages_path': harble_messages_path}
+                host, port, hotel_version, client_identifier, client_type = packet.read("sisss")
+                self.__parse_packet_infos(packet)
 
-                if harble_messages_path != '' and harble_messages_path != 'null':
-                    self.__harble_api_init()
+                self.connection_info = {'host': host, 'port': port, 'hotel_version': hotel_version,
+                                        'client_identifier': client_identifier, 'client_type': client_type}
 
                 self.__raise_event('connection_start')
 
+                if self.__await_connect_packet:
+                    self.__await_connect_packet = False
+                    self.__start_barrier.wait()
+
             elif message_type == INCOMING_MESSAGES.CONNECTION_END:
                 self.__raise_event('connection_end')
                 self.connection_info = None
-                self.harble_api = None
+                self.packet_infos = None
 
             elif message_type == INCOMING_MESSAGES.FLAGS_CHECK:
                 size = packet.read_int()
@@ -225,7 +232,10 @@ class Extension:
                     'green',
                     False
                 )
-                self.__start_barrier.wait()
+
+                self.__await_connect_packet = packet.read_bool()
+                if not self.__await_connect_packet:
+                    self.__start_barrier.wait()
 
             elif message_type == INCOMING_MESSAGES.ON_DOUBLE_CLICK:
                 self.__raise_event('double_click')
@@ -249,53 +259,72 @@ class Extension:
                 self.__response = HPacket.reconstruct_from_java(packet_string)
                 self.__response_barrier.wait()
 
-    def __harble_api_init(self):
-        try:
-            path = self.connection_info['harble_messages_path']
-            with open(path) as f:
+    def __parse_packet_infos(self, packet : HPacket):
+        incoming = {}
+        outgoing = {}
 
-                def generate_harble_dict(json_list):
-                    dict = {}
-                    for elem in json_list:
-                        dict[elem['Id']] = elem
-                        dict[elem['Hash']] = elem
-                        name = elem['Name']
-                        if name is not None and name != '' and name != 'null':
-                            dict[name] = elem
-                    return dict
+        length = packet.read_int()
+        for _ in range(length):
+            headerId, hash, name, structure, isOutgoing, source = packet.read('isssBs')
+            name = name if name is not 'NULL' else None
+            hash = hash if hash is not 'NULL' else None
+            structure = structure if structure is not 'NULL' else None
 
-                harble_api_json = json.load(f)
-                incoming_json = harble_api_json['Incoming']
-                outgoing_json = harble_api_json['Outgoing']
-                incoming = generate_harble_dict(incoming_json)
-                outgoing = generate_harble_dict(outgoing_json)
+            elem = {'Id': headerId, 'Name': name, 'Hash': hash, 'Structure': structure, 'Source': source}
+
+            packet_dict = outgoing if isOutgoing else incoming
+            if headerId not in packet_dict:
+                packet_dict[headerId] = []
+            packet_dict[headerId].append(elem)
+
+            if hash is not None:
+                if hash not in packet_dict:
+                    packet_dict[hash] = []
+                packet_dict[hash].append(elem)
+
+            if name is not None:
+                if name not in packet_dict:
+                    packet_dict[name] = []
+                packet_dict[name].append(elem)
+
+        self.packet_infos = {Direction.TO_CLIENT: incoming, Direction.TO_SERVER: outgoing}
 
-                self.harble_api = {Direction.TO_CLIENT: incoming, Direction.TO_SERVER: outgoing}
-        except:
-            self.harble_api = None
-            self.write_to_console('Failed parsing HarbleAPI', 'red')
 
     def __send_to_stream(self, packet):
         self.__stream_lock.acquire()
         self.__sock.send(packet.bytearray)
         self.__stream_lock.release()
 
+    def __callbacks(self, callbacks):
+        for func in callbacks:
+            func()
+
     def __raise_event(self, event_name):
         if event_name in self.__events:
-            for func in self.__events[event_name]:
-                func()
+            t = threading.Thread(target=self.__callbacks, args=(self.__events[event_name],))
+            t.start()
 
     def __send(self, direction, packet: HPacket):
         if not self.is_closed():
 
             old_settings = None
-            if packet.is_harble_api_packet():
-                old_settings = (packet.header_id(), packet.is_edited, packet.harble_id)
+            if packet.is_incomplete_packet():
+                old_settings = (packet.header_id(), packet.is_edited, packet.incomplete_identifier)
                 packet.fill_id(direction, self)
 
+            if self.connection_info == None:
+                self.__lost_packets += 1
+                print("Could not send packet because G-Earth isn't connected to a client", file=sys.stderr)
+                return False
+
             if packet.is_corrupted():
                 self.__lost_packets += 1
-                print('Could not send corrupted packet')
+                print('Could not send corrupted', file=sys.stderr)
+                return False
+
+            if packet.is_incomplete_packet():
+                self.__lost_packets += 1
+                print('Could not send incomplete packet', file=sys.stderr)
                 return False
 
             wrapper_packet = HPacket(OUTGOING_MESSAGES.SEND_MESSAGE.value, direction == Direction.TO_SERVER,
@@ -304,7 +333,7 @@ class Extension:
 
             if old_settings is not None:
                 packet.replace_short(4, old_settings[0])
-                packet.harble_id = old_settings[2]
+                packet.incomplete_identifier = old_settings[2]
                 packet.is_edited = old_settings[1]
 
             return True
