@@ -5,12 +5,12 @@ class HPacket:
     default_extension = None
 
     def __init__(self, id, *objects):
-        self.harble_id = None if (type(id) is int) else id
+        self.incomplete_identifier = None if (type(id) is int) else id
 
         self.read_index = 6
         self.bytearray = bytearray(b'\x00\x00\x00\x02\xff\xff')
-        if self.harble_id is None:
-            self.replace_ushort(4, id)
+        if self.incomplete_identifier is None:
+            self.replace_short(4, id)
         self.is_edited = False
 
         for object in objects:
@@ -26,17 +26,17 @@ class HPacket:
         self.is_edited = False
 
     def fill_id(self, direction: Direction, extension=None) -> bool:
-        if self.harble_id is not None:
+        if self.incomplete_identifier is not None:
             if extension is None:
                 if self.default_extension is None:
                     return False
                 extension = self.default_extension
 
-            if extension.harble_api is not None and self.harble_id in extension.harble_api[direction]:
+            if extension.packet_infos is not None and self.incomplete_identifier in extension.packet_infos[direction]:
                 edited_old = self.is_edited
-                self.replace_ushort(4, extension.harble_api[direction][self.harble_id]['Id'])
+                self.replace_short(4, extension.packet_infos[direction][self.incomplete_identifier][0]['Id'])
                 self.is_edited = edited_old
-                self.harble_id = None
+                self.incomplete_identifier = None
                 return True
             return False
         return True
@@ -68,7 +68,7 @@ class HPacket:
 
         obj.bytearray = bytearray(string[1:].encode("iso-8859-1"))
         obj.is_edited = string[0] == '1'
-        obj.harble_id = None
+        obj.incomplete_identifier = None
         return obj
 
     def __repr__(self):
@@ -81,11 +81,11 @@ class HPacket:
         return self.read_int(0)
 
     def __str__(self):
-        return "(id:{}, length:{}) -> {}".format(self.header_id() if not self.is_harble_api_packet() else self.harble_id,
+        return "(id:{}, length:{}) -> {}".format(self.header_id() if not self.is_incomplete_packet() else self.incomplete_identifier,
                                                  len(self), bytes(self))
 
-    def is_harble_api_packet(self) -> bool:
-        return self.harble_id is not None
+    def is_incomplete_packet(self) -> bool:
+        return self.incomplete_identifier is not None
 
     def g_string(self, extension=None) -> str:
         if extension is None:
@@ -112,7 +112,7 @@ class HPacket:
         self.read_index = 6
 
     def header_id(self) -> int:
-        return self.read_ushort(4)
+        return self.read_short(4)
 
     def fix_length(self) -> None:
         self.replace_int(0, len(self.bytearray) - 4)
@@ -124,12 +124,19 @@ class HPacket:
 
         return int.from_bytes(self.bytearray[index:index + 4], byteorder='big', signed=True)
 
-    def read_ushort(self, index=None) -> int:
+    def read_short(self, index=None) -> int:
         if index is None:
             index = self.read_index
             self.read_index += 2
 
-        return int.from_bytes(self.bytearray[index:index + 2], byteorder='big', signed=False)
+        return int.from_bytes(self.bytearray[index:index + 2], byteorder='big', signed=True)
+
+    def read_long(self, index=None) -> int:
+        if index is None:
+            index = self.read_index
+            self.read_index += 8
+
+        return int.from_bytes(self.bytearray[index:index + 8], byteorder='big', signed=True)
 
     def read_string(self, index=None, head=2, encoding='iso-8859-1') -> str:
         if index is None:
@@ -161,7 +168,9 @@ class HPacket:
             'i': self.read_int,
             's': self.read_string,
             'b': self.read_byte,
-            'B': self.read_bool
+            'B': self.read_bool,
+            'u': self.read_short,
+            'l': self.read_long
         }
         return [read_methods[value_type]() for value_type in structure]
 
@@ -169,8 +178,12 @@ class HPacket:
         self.bytearray[index:index + 4] = value.to_bytes(4, byteorder='big', signed=True)
         self.is_edited = True
 
-    def replace_ushort(self, index, value) -> None:
-        self.bytearray[index:index + 2] = value.to_bytes(2, byteorder='big', signed=False)
+    def replace_short(self, index, value) -> None:
+        self.bytearray[index:index + 2] = value.to_bytes(2, byteorder='big', signed=True)
+        self.is_edited = True
+
+    def replace_long(self, index, value) -> None:
+        self.bytearray[index:index + 8] = value.to_bytes(8, byteorder='big', signed=False)
         self.is_edited = True
 
     def replace_bool(self, index, value) -> None:
@@ -178,7 +191,7 @@ class HPacket:
         self.is_edited = True
 
     def replace_string(self, index, value, encoding='utf-8') -> None:
-        old_len = self.read_ushort(index)
+        old_len = self.read_short(index)
         part1 = self.bytearray[0:index]
         part3 = self.bytearray[index + 2 + old_len:]
 
@@ -196,8 +209,14 @@ class HPacket:
         self.is_edited = True
         return self
 
-    def append_ushort(self, value):
-        self.bytearray.extend(value.to_bytes(2, byteorder='big', signed=False))
+    def append_short(self, value):
+        self.bytearray.extend(value.to_bytes(2, byteorder='big', signed=True))
+        self.fix_length()
+        self.is_edited = True
+        return self
+
+    def append_long(self, value):
+        self.bytearray.extend(value.to_bytes(8, byteorder='big', signed=False))
         self.fix_length()
         self.is_edited = True
         return self
